@@ -3,7 +3,7 @@
 #'
 #'   Utilities for text output, etc
 #'
-#'   $Revision: 1.11 $ $Date: 2023/02/28 02:00:22 $
+#'   $Revision: 1.16 $ $Date: 2025/11/19 03:14:45 $
 #'
 
 # text magic
@@ -225,11 +225,22 @@ ordinalsuffix <- function(k) {
   return(ending)
 }
 
-articlebeforenumber <- function(k) {
-  k <- abs(k)
-  if(k == 11 || k == 18) return("an")
+articlebeforenumber <- function(k, teenhundreds=FALSE) {
+  if(k < 0) return("a")  
+  k <- floor(k)
+  if(k == 0) return("a")
+  ## leading digit
   leading <- floor(k/10^floor(log10(k)))
   if(leading == 8) return("an")
+  ## leading digits in front of power of 1000
+  kilo <- floor(k/10^(3*floor(log10(k)/3)))
+  if(kilo == 11 || kilo == 18) return("an")
+  ## 
+  if(teenhundreds) {
+    hundreds <- floor(k/100)
+    if(hundreds == 11 || hundreds == 18)
+      return("an")
+  }
   return("a")
 }
 
